/*	Copyright (C) 1996, 1997 Free Software Foundation, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * As a special exception, the Free Software Foundation gives permission
 * for additional uses of the text contained in its release of GUILE.
 *
 * The exception is that, if you link the GUILE library with other files
 * to produce an executable, this does not by itself cause the
 * resulting executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of
 * linking the GUILE library code into it.
 *
 * This exception does not however invalidate any other reasons why
 * the executable file might be covered by the GNU General Public License.
 *
 * This exception applies only to the code released by the
 * Free Software Foundation under the name GUILE.  If you copy
 * code from other Free Software Foundation releases into a copy of
 * GUILE, as the General Public License permits, the exception does
 * not apply to the code that you add in this way.  To avoid misleading
 * anyone as to the status of such modified files, you must delete
 * this exception notice from them.
 *
 * If you write modifications of your own for GUILE, it is your choice
 * whether to permit this exception to apply to your modifications.
 * If you do not wish that, delete this exception notice.  
 */

#include <libguile.h>

/* Debugger interface (don't change the order of the following lines) */
#define GDB_TYPE SCM
#include <libguile/gdb_interface.h>
GDB_INTERFACE;

extern int scm_boot_guile_1_live;
extern int scm_ice_9_already_loaded;

void
scm_prepare_dump ()
{
    scm_boot_guile_1_live = 0;
}

void
scm_init_after_dump ()
{
    scm_ice_9_already_loaded = 1;
    scm_init_load_path ();
}

static void
dodump (char *progname, char *dumpfile)
{
  scm_prepare_dump ();
  scm_igc ("dump");
  sbrk (4096);  /* work around a bug somewhere */
  unexec (dumpfile, progname, 0, 0, 0);
  printf ("dumped `%s'.\n", dumpfile);
}
  
static void
inner_main (void *closure, int argc, char **argv)
{
  static int dumped = 0;

  /* module initializations would go here */

  if (argc >= 3 && !strcmp (argv[1], "--dump"))
    {
      int i;

      scm_primitive_load_path (scm_makfrom0str ("ice-9/boot-9.scm"));
      for (i = 3; i < argc; i++)
	{
	  printf ("loading `%s'\n", argv[i]);
	  scm_primitive_load (scm_makfrom0str (argv[i]));
	}
      dumped = 1;
      dodump (argv[0], argv[2]);
    }
  else
    {
      if (dumped)
	scm_init_after_dump ();
      scm_shell (argc, argv);
    }
}

int
main (int argc, char **argv)
{
  scm_boot_guile (argc, argv, inner_main, 0);
  return 0; /* never reached */
}
